// Graphing sketch

// This program takes ASCII-encoded strings from the serial port at 9600 baud
// and graphs them. It expects values in the range 0 to 1023, followed by a
// newline, or newline and carriage return

// created 20 Apr 2005
// updated 24 Nov 2015
// by Tom Igoe
// updated 19 May 2021
// by Mohit Ahuja 
// This example code is in the public domain.

import processing.serial.*;

Serial serial_port;        // The serial port
int xPos = 1;         // horizontal position of the graph
float inByte = 0;

// serial port buttons
Button btn_serial_up;              // move up through the serial port list
Button btn_serial_dn;              // move down through the serial port list
Button btn_serial_connect;         // connect to the selected serial port
Button btn_serial_disconnect;      // disconnect from the serial port
Button btn_serial_list_refresh;    // refresh the serial port list
Button btn_baud_up;
Button btn_baud_dn;
Button btn_baud_sel;

String baud[]={"300", "400", "1200", "2400", "4800", "9600", "19200", "38400", "57600", "74880", "115200", "230400"};
String baudrate = baud[4];
int baud_index = 0;

String typing = "";                // Variable to store text currently being typed
String saved = "";                 // Variable to store saved text when return is hit

String serial_list;                // list of serial ports
int serial_list_index = 0;         // currently selected serial port 
int num_serial_ports = 0;          // number of serial ports in the list

int connected =0;
int factor=1;
int h=300;

void setup () {
  // set the window size:
  size(700, 700);

  // create the buttons
  btn_serial_up = new Button("^", 140, 10, 40, 20);
  btn_serial_dn = new Button("v", 140, 30, 40, 20);
  btn_serial_connect = new Button("Connect", 190, 10, 80, 40);
  btn_serial_disconnect = new Button("Disconnect", 280, 10, 80, 40);
  btn_serial_list_refresh = new Button("Refresh", 370, 10, 100, 40);

  btn_baud_up = new Button("^", 140, 60, 40, 20);
  btn_baud_dn = new Button("v", 140, 80, 40, 20);

  // get the list of serial ports on the computer
  serial_list = Serial.list()[serial_list_index];

  // List all the available serial ports
  // if using Processing 2.1 or later, use Serial.printArray()
  println(Serial.list());

  // get the number of serial ports in the list
  num_serial_ports = Serial.list().length;

  // set initial background:
  background(0);
}

void mousePressed() {
  // up button clicked
  if (btn_serial_up.MouseIsOver()) {
    if (serial_list_index > 0) {
      // move one position up in the list of serial ports
      serial_list_index--;
      serial_list = Serial.list()[serial_list_index];
    }
  }
  // down button clicked
  if (btn_serial_dn.MouseIsOver()) {
    if (serial_list_index < (num_serial_ports - 1)) {
      // move one position down in the list of serial ports
      serial_list_index++;
      serial_list = Serial.list()[serial_list_index];
    }
  }
  // Connect button clicked
  if (btn_serial_connect.MouseIsOver()) {
    if (serial_port == null) {
      // connect to the selected serial port
      serial_port = new Serial(this, Serial.list()[serial_list_index], Integer.valueOf(baudrate));

      // don't generate a serialEvent() unless you get a newline character:
      serial_port.bufferUntil('\n');

      connected=1;

      fill(0);
      rect(10, 100, 480, 20);
      fill(255);
      text(serial_list+" connected at "+baudrate+" baudrate and amplification "+saved, 10, 115);
    }
  }
  // Disconnect button clicked
  if (btn_serial_disconnect.MouseIsOver()) {
    if (serial_port != null) {
      // disconnect from the serial port
      serial_port.stop();
      serial_port = null;
      connected=0;
      fill(0);
      rect(10, 100, 480, 20);
      fill(255);
      text(serial_list+" diconnected", 10, 115);
    }
  }
  // Refresh button clicked
  if (btn_serial_list_refresh.MouseIsOver()) {
    // get the serial port list and length of the list
    serial_list = Serial.list()[serial_list_index];
    num_serial_ports = Serial.list().length;
  }
  // up button clicked
  if (btn_baud_up.MouseIsOver()) {
    if (baud_index > 0) {
      // move one position up in the list of serial ports
      baud_index--;
      baudrate = baud[baud_index];
    }
  }
  // down button clicked
  if (btn_baud_dn.MouseIsOver()) {
    if (baud_index < (baud.length - 1)) {
      // move one position down in the list of serial ports
      baud_index++;
      baudrate = baud[baud_index];
    }
  }
}
void draw () {
  // draw the buttons in the application window
  btn_serial_up.Draw();
  btn_serial_dn.Draw();
  btn_serial_connect.Draw();
  btn_serial_disconnect.Draw();
  btn_serial_list_refresh.Draw();

  btn_baud_up.Draw();
  btn_baud_dn.Draw();

  // draw the text box containing the selected serial port
  DrawTextBox(serial_list, 10, 10, 120, 40);
  DrawTextBox(baudrate, 10, 60, 120, 40);

  int indent=18;
  fill(255);
  rect(10, 135, 400, 20);
  fill(0);
  text("Amplification factor: " + typing, indent, 150);

  stroke(50, 50, 255);
  line(0, height-h-100, width, height-h-100);
  stroke(50, 50, 255);
  line(0, height-h, width, height-h);
  stroke(50, 50, 255);
  line(0, height-h+100, width, height-h+100);

  // draw the line:
  if (connected==1) {
    stroke(255);
    line(xPos, height, xPos, height - inByte);
    stroke(0);
    line(xPos, height+2, xPos, height - inByte+2);

    stroke(50, 50, 255);
    line(0, height-h-100, width, height-h-100);
    stroke(50, 50, 255);
    line(0, height-h, width, height-h);
    stroke(50, 50, 255);
    line(0, height-h+100, width, height-h+100);

    // at the edge of the screen, go back to the beginning:
    if (xPos >= width) {
      xPos = 0;
      background(0);
    } else {
      // increment the horizontal position:
      xPos++;
    }
  }
}

void serialEvent (Serial serial_port) {
  // get the ASCII string:
  String inString = serial_port.readStringUntil('\n');
  if (inString != null) {
    // trim off any whitespace:
    inString = trim(inString);
    // convert to an int and map to the screen height:
    inByte = float(inString);
    inByte = map(inByte, 0, 1023, 0, 1000*factor)+h;
    println(inByte);
  }
}

void keyPressed() {
  // If the return key is pressed, save the String and clear it
  if (key == '\n' ) {
    saved = typing;
    factor=Integer.valueOf(saved);
    // A String can be cleared by setting it equal to ""
    typing = "";
    fill(0);
    rect(10, 100, 480, 20);
    fill(255);
    text(serial_list+" connected at "+baudrate+" baudrate and amplification "+saved, 10, 115);
  } else if (key==BACKSPACE) {
    typing = typing.substring( 0, typing.length()-1 );
  } else switch(key) {
  case '1':
  case '2':
  case '3':
  case '4':
  case '5':
  case '6':
  case '7':
  case '8':
  case '9':
  case '0':
    // Otherwise, concatenate the String
    // Each character typed by the user is added to the end of the String variable.
    typing = typing + key;
    break;
  }
}

// function for drawing a text box with title and contents
void DrawTextBox(String str, int x, int y, int w, int h)
{
  fill(255);
  rect(x, y, w, h);
  fill(0);
  textAlign(LEFT);
  textSize(12);  
  text(str, x + 10, y + 15, w - 20, h - 10);
}

// button class used for all buttons
class Button {
  String label;
  float x;    // top left corner x position
  float y;    // top left corner y position
  float w;    // width of button
  float h;    // height of button

  // constructor
  Button(String labelB, float xpos, float ypos, float widthB, float heightB) {
    label = labelB;
    x = xpos;
    y = ypos;
    w = widthB;
    h = heightB;
  }

  // draw the button in the window
  void Draw() {
    fill(218);
    stroke(141);
    rect(x, y, w, h, 10);
    textAlign(CENTER, CENTER);
    fill(0);
    text(label, x + (w / 2), y + (h / 2));
  }

  // returns true if the mouse cursor is over the button
  boolean MouseIsOver() {
    if (mouseX > x && mouseX < (x + w) && mouseY > y && mouseY < (y + h)) {
      return true;
    }
    return false;
  }
}
